"""django-ckeditors app.py file"""
from __future__ import annotations

from django.apps import AppConfig
from django.conf import settings


class DjangoCKEditorsConfig(AppConfig):
    name = "django_ckeditors"

    def ready(self):
        # Unsure as to why this was used originally,
        # but here it is just in case.
        if not hasattr(settings, "DJ_CKE_CSRF_COOKIE_NAME"):
            settings.DJ_CKE_CSRF_COOKIE_NAME: str = settings.CSRF_COOKIE_NAME  # type: ignore[attr-defined]

        if not hasattr(settings, "DJ_CKE_CUSTOM_COLOUR_PALETTE"):
            settings.DJ_CKE_CUSTOM_COLOUR_PALETTE: list = [  # type: ignore[attr-defined]
                {"color": "hsl(4, 90%, 58%)", "label": "Red"},
                {"color": "hsl(340, 82%, 52%)", "label": "Pink"},
                {"color": "hsl(291, 64%, 42%)", "label": "Purple"},
                {"color": "hsl(262, 52%, 47%)", "label": "Deep Purple"},
                {"color": "hsl(231, 48%, 48%)", "label": "Indigo"},
                {"color": "hsl(207, 90%, 54%)", "label": "Blue"},
            ]

        if not hasattr(settings, "DJ_CKE_CUSTOM_CSS"):
            settings.DJ_CKE_CUSTOM_CSS: str = ""  # type: ignore[attr-defined]

        if not hasattr(settings, "DJ_CKE_IMAGE_FORMATTER"):
            settings.DJ_CKE_IMAGE_FORMATTER = "django_ckeditors.image.image_formatter"  # type: ignore[attr-defined]

        if not hasattr(settings, "DJ_CKE_IMAGE_URL_HANDLER"):
            settings.DJ_CKE_IMAGE_URL_HANDLER: str = ""  # type: ignore[attr-defined]

        if not hasattr(settings, "DJ_CKE_IMAGE_STORAGE"):
            settings.DJ_CKE_IMAGE_STORAGE: str = ""  # type: ignore[attr-defined]

        if not hasattr(settings, "DJ_CKE_PERMITTED_IMAGE_TYPES"):
            settings.DJ_CKE_PERMITTED_IMAGE_TYPES: list[str] = [  # type: ignore[attr-defined]
                "jpeg",
                "png",
                "gif",
                "bmp",
                "webp",
                "tiff",
            ]

        if not hasattr(settings, "DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS"):
            settings.DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS: bool = False  # type: ignore[attr-defined]

        if not hasattr(settings, "DJ_CKE_EDITORS_CONFIGS"):
            settings.DJ_CKE_EDITORS_CONFIGS: dict = {  # type: ignore[attr-defined]
                "default": {
                    "toolbar": [
                        "heading",
                        "|",
                        "bold",
                        "italic",
                        "link",
                        "bulletedList",
                        "numberedList",
                        "blockQuote",
                        "imageUpload",
                    ],
                },
                "extends": {
                    "blockToolbar": [
                        "paragraph",
                        "heading1",
                        "heading2",
                        "heading3",
                        "|",
                        "bulletedList",
                        "numberedList",
                        "|",
                        "blockQuote",
                    ],
                    "toolbar": [
                        "heading",
                        "|",
                        "outdent",
                        "indent",
                        "|",
                        "bold",
                        "italic",
                        "link",
                        "underline",
                        "strikethrough",
                        "code",
                        "subscript",
                        "superscript",
                        "highlight",
                        "|",
                        "codeBlock",
                        "sourceEditing",
                        "insertImage",
                        "bulletedList",
                        "numberedList",
                        "todoList",
                        "|",
                        "blockQuote",
                        "imageUpload",
                        "|",
                        "fontSize",
                        "fontFamily",
                        "fontColor",
                        "fontBackgroundColor",
                        "mediaEmbed",
                        "removeFormat",
                        "insertTable",
                    ],
                    "image": {
                        "toolbar": [
                            "imageTextAlternative",
                            "|",
                            "imageStyle:alignLeft",
                            "imageStyle:alignRight",
                            "imageStyle:alignCenter",
                            "imageStyle:side",
                            "|",
                        ],
                        "styles": [
                            "full",
                            "side",
                            "alignLeft",
                            "alignRight",
                            "alignCenter",
                        ],
                    },
                    "table": {
                        "contentToolbar": [
                            "tableColumn",
                            "tableRow",
                            "mergeTableCells",
                            "tableProperties",
                            "tableCellProperties",
                        ],
                        "tableProperties": {
                            "borderColors": settings.DJ_CKE_CUSTOM_COLOUR_PALETTE,
                            "backgroundColors": settings.DJ_CKE_CUSTOM_COLOUR_PALETTE,
                        },
                        "tableCellProperties": {
                            "borderColors": settings.DJ_CKE_CUSTOM_COLOUR_PALETTE,
                            "backgroundColors": settings.DJ_CKE_CUSTOM_COLOUR_PALETTE,
                        },
                    },
                    "heading": {
                        "options": [
                            {
                                "model": "paragraph",
                                "title": "Paragraph",
                                "class": "ck-heading_paragraph",
                            },
                            {
                                "model": "heading1",
                                "view": "h1",
                                "title": "Heading 1",
                                "class": "ck-heading_heading1",
                            },
                            {
                                "model": "heading2",
                                "view": "h2",
                                "title": "Heading 2",
                                "class": "ck-heading_heading2",
                            },
                            {
                                "model": "heading3",
                                "view": "h3",
                                "title": "Heading 3",
                                "class": "ck-heading_heading3",
                            },
                        ],
                    },
                },
                "list": {
                    "properties": {
                        "styles": "true",
                        "startIndex": "true",
                        "reversed": "true",
                    },
                },
            }



"""blog custom storage module."""
from django.core.files.storage import FileSystemStorage


class ImageCustomFileSystemStorage(FileSystemStorage):
    """
    Overrides save and url methods to add a custom path prefix to
    file URLs for your testing environment.
    """

    def save(self, name, content, max_length=None):
        """
        Overrides the save method. We'll leave the saving logic to the base
        FileSystemStorage class.
        """
        new_name = f"tests/custom/path/{name}"
        return super().save(
            name=new_name,
            content=content,
            max_length=max_length,
        )

    def url(self, name):
        """Overrides the URL generation for saved files."""
        return name


'''
# def image_url_handler(request):
#     """Return a unique image path_to/filename"""
#     img = request.FILES["upload"]
#     fpath = pathlib.Path(img.name)
#     new_name = "new_img_name"
#
#     return f"images/{new_name}{fpath.suffix}"
'''



import os
from urllib.parse import urljoin

from django.conf import settings
from django.core.files.storage import FileSystemStorage


class CustomStorage(FileSystemStorage):
    """Custom storage for django_ckeditors images."""

    location = os.path.join(settings.MEDIA_ROOT, "django_ckeditors")
    base_url = urljoin(settings.MEDIA_URL, "django_ckeditors/")



class PillowImageError(Exception):
    """Catchall for when a Pillow read error is encountered.

    """

    def __init__(self, message, image_type=None):
        super().__init__(message)
        self.image_type = image_type


class InvalidImageTypeError(Exception):
    """Raised when an unsupported image type is encountered."""

    def __init__(self, message, image_type=None):
        super().__init__(message)
        self.image_type = image_type



"""django-ckeditor helpers."""

import logging

import filetype
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import import_string
from PIL import Image, UnidentifiedImageError
from PIL.Image import DecompressionBombError

from django_ckeditors.exceptions import (
    InvalidImageTypeError,
    PillowImageError,
)

logger = logging.getLogger(__name__)


def get_storage_class():
    """
    Determines the appropriate storage class for CKEditors based on settings.

    This function searches through a prioritized set of Django settings
    to dynamically determine the storage class to be used.

    Priority Order:
        1. DJ_CKE_FILE_STORAGE setting.
        2. DEFAULT_FILE_STORAGE
        3. STORAGES['default']

    :returns The imported storage class:

    :raises ImproperlyConfigured: If no valid storage class configuration is found.
    """
    # We can directly call DJ_CKE_IMAGE_STORAGE because it is always available.
    dj_cke_img_storage_setting = settings.DJ_CKE_IMAGE_STORAGE
    default_storage_setting = getattr(settings, "DEFAULT_FILE_STORAGE", None)
    storages_setting = getattr(settings, "STORAGES", {})
    default_storage_name = storages_setting.get("default", {}).get("BACKEND")

    storage_class: str = ""

    # Explanation: import_string allows us to import a class dynamically
    # based on its name as a string within settings.

    if dj_cke_img_storage_setting:
        storage_class = dj_cke_img_storage_setting
    elif default_storage_setting:
        storage_class = default_storage_setting
    elif default_storage_name:
        storage_class = default_storage_name
    try:
        return _get_storage_object(storage_class)
    except ImproperlyConfigured as e:
        logger.exception({e.args[0]})
        raise ImproperlyConfigured from e


def _get_storage_object(storage_class: str = ""):
    try:
        storage = import_string(storage_class)
        return storage()
    except ImportError as e:
        error_msg = (
            "Either DJ_CKE_IMAGE_STORAGE, DEFAULT_FILE_STORAGE, "
            "or STORAGES['default'] setting is required."
        )
        raise ImproperlyConfigured(error_msg) from e


def image_verify(image):
    """Verifies whether an image file is valid and has a supported type.

    Validates an image file and ensures it falls within the permitted image
    types. The function checks for potential corruption, decompression bombs,
    and unsupported file formats.

    :param image: The image file to verify. An image file-like object or a filename.
    :type image: file-like object or str
    :raises PillowImageError: If the image is corrupt, too large, or cannot be verified.
    :raises InvalidImageTypeError: If the image has an unsupported file type.
    """

    # Fallback to `CKEditor 5` default image types if not set.
    permitted_image_types = settings.DJ_CKE_PERMITTED_IMAGE_TYPES

    kind = filetype.guess(image)
    if kind is None or kind.extension not in permitted_image_types:
        error_msg = (
            "Invalid image type, valid types "
            "[ 'jpeg', 'png', 'gif', 'bmp', 'webp', 'tiff' ]"
        )
        logger.error(error_msg)
        raise InvalidImageTypeError(error_msg)


    try:
        Image.open(image).verify()
    except FileNotFoundError as e:
        error_msg = "This image file is not valid or corrupted."
        logger.exception(
            error_msg,
        )
        raise PillowImageError(error_msg, e) from e
    except UnidentifiedImageError as e:
        error_msg = "This image file is corrupted."
        logger.exception(
            error_msg,
        )
        raise PillowImageError(error_msg, e) from e
    except DecompressionBombError as e:
        error_msg = "This image file is corrupted or to large to use."
        logger.exception(
            error_msg,
        )
        raise PillowImageError(error_msg, e) from e


def handle_uploaded_image(request):
    """Handles an uploaded image, saving it to storage and returning its URL.

    Leverages a custom URL handler if specified in Django settings.

    :param request: (HttpRequest) The Django request object containing the
        uploaded file.
        Available in `request.FILES["upload"]`

    :returns: (URL) The URL where the uploaded image is stored.
    """

    try:
        storage = get_storage_class()
    except ImproperlyConfigured:
        error_msg = "A valid storage system has not been configured"
        logger.exception(error_msg)
        # .. todo:: Return json response with errors.
        return error_msg

    img = request.FILES["upload"]  # Extract the uploaded image file

    # Check for a custom URL handler in Django setting
    custom_url_handler = getattr(settings, "DJ_CKE_IMAGE_URL_HANDLER", None)
    if custom_url_handler:
        url_handler = import_string(custom_url_handler)
        url = url_handler(request)  # Get the URL using the custom handler

    else:

        url = img.name  # Default to using the image's filename as the URL

    if settings.DJ_CKE_IMAGE_FORMATTER:
        try:
            formatter = import_string(settings.DJ_CKE_IMAGE_FORMATTER)
            img=formatter(img)
            print('^^^^^^^^^^  BACK IN HELPER IMAGE STUFF WORKED')
        except TypeError as e:
            raise TypeError from e
    print('PRE FILENAME')
    filename = storage.save(name=url, content=img)
    print('POST FILENAME')
    return storage.url(filename)  # Return the URL of the saved image


def has_permission_to_upload_images(request) -> bool:

    has_perms = True
    if (
        hasattr(settings, "DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS")
        and (settings.DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS)
        and not request.user.is_staff
    ):
        has_perms = False

    return has_perms



from django import forms

from django_ckeditors.widgets import CKEditorsWidget

from .models import Article, Comment


class CommentForm(forms.ModelForm):
    """Custom storage for django_ckeditors images."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["text"].required = False

    class Meta:
        model = Comment
        fields = ("author", "text")
        widgets = {
            "text": CKEditorsWidget(
                attrs={"class": "django_ckeditors"},
                config_name="comment",
            ),
        }


class ArticleForm(forms.ModelForm):
    class Meta:
        model = Article
        fields = ["title", "text"]
        widgets = {
            "text": CKEditorsWidget(
                attrs={"class": "django_ckeditors"},
                config_name="comment",
            ),
            "text2": CKEditorsWidget(
                attrs={"class": "django_ckeditors"},
                config_name="comment",
            ),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["text"].required = False



# import pytest
# from django.core.exceptions import ImproperlyConfigured
# from django.test import override_settings
# from django.utils.module_loading import import_string
#
# from django_ckeditors.helpers import get_storage_class
#
#
# @override_settings(
#     DJ_CKE_IMAGE_STORAGE="articles.storage.CustomStorage",
#     DEFAULT_FILE_STORAGE="django.core.files.storage.FileSystemStorage",
#     STORAGES={"default": {"BACKEND": "django.core.files.storage.FileSystemStorage"}},
# )
# def test_get_storage_class(settings):
#     # Case 1: DJ_CKE_IMAGE_STORAGE is defined
#     storage_class = get_storage_class()
#     assert storage_class == import_string(settings.DJ_CKE_IMAGE_STORAGE)
#
#     # Case 2: DEFAULT_FILE_STORAGE is defined
#     delattr(settings, "DJ_CKE_IMAGE_STORAGE")
#     storage_class = get_storage_class()
#     assert storage_class == import_string(settings.DEFAULT_FILE_STORAGE)
#
#     # Case 3: STORAGES['default'] is defined
#     delattr(settings, "DEFAULT_FILE_STORAGE")
#     storage_class = get_storage_class()
#     assert storage_class == import_string(settings.STORAGES["default"]["BACKEND"])
#
#     # Case 4: None of the required settings is defined
#     delattr(settings, "STORAGES")
#     with pytest.raises(ImproperlyConfigured):
#         get_storage_class()



from unittest.mock import patch  # Import the 'patch' decorator

from django.conf import settings
from django.test import SimpleTestCase

from django_ckeditors.apps import DjangoCKEditorsConfig


class TestDjangoCKEditorsSettingsConfig(SimpleTestCase):
    def test_dj_cke_csrf_cookie_name_defaults_to_csrf_cookie_name(self):
        # default setting
        assert settings.DJ_CKE_CSRF_COOKIE_NAME == settings.CSRF_COOKIE_NAME

    def test_dj_cke_csrf_cookie_name_uses_user_override(self):
        # user setting
        with patch.object(settings, "DJ_CKE_CSRF_COOKIE_NAME", "user_cookie_name"):
            DjangoCKEditorsConfig.ready(self)
            assert settings.DJ_CKE_CSRF_COOKIE_NAME == "user_cookie_name"

    def test_dj_cke_custom_colour_pallette_default_setting(self):

        # default setting
        assert "color" in list(
            settings.DJ_CKE_CUSTOM_COLOUR_PALETTE[0].keys(),
        )
        assert "label" in list(
            settings.DJ_CKE_CUSTOM_COLOUR_PALETTE[0].keys(),
        )

    def test_dj_cke_custom_colour_pallette_uses_user_override(self):
        # user setting
        with patch.object(
            settings,
            "DJ_CKE_CUSTOM_COLOUR_PALETTE",
            [{"color": "hsl(207, 90%, 54%)", "label": "Blue"}],
        ):
            DjangoCKEditorsConfig.ready(self)
            assert [
                {"color": "hsl(207, 90%, 54%)", "label": "Blue"},
            ] == settings.DJ_CKE_CUSTOM_COLOUR_PALETTE

    def test_dj_cke_custom_css_default_setting(self):
        # default setting
        assert settings.DJ_CKE_CUSTOM_CSS == ""

    def test_dj_cke_custom_css_uses_user_override(self):
        # user setting
        with patch.object(settings, "DJ_CKE_CUSTOM_CSS", "custom.css"):
            DjangoCKEditorsConfig.ready(self)
            assert settings.DJ_CKE_CUSTOM_CSS == "custom.css"

    def test_dj_cke_image_url_handler_default_setting(self):
        # default setting
        assert settings.DJ_CKE_IMAGE_URL_HANDLER == ""

    def test_dj_cke_image_url_handler_uses_user_override(self):
        # user setting
        with patch.object(settings, "DJ_CKE_IMAGE_URL_HANDLER", "custom.image.handler"):
            DjangoCKEditorsConfig.ready(self)
            assert settings.DJ_CKE_IMAGE_URL_HANDLER == "custom.image.handler"

    def test_dj_cke_image_storage_default_setting(self):
        # default setting
        assert settings.DJ_CKE_IMAGE_STORAGE == ""

    def test_dj_cke_image_storage_uses_user_override(self):
        # user setting
        with patch.object(settings, "DJ_CKE_IMAGE_STORAGE", "custom.image.storage"):
            DjangoCKEditorsConfig.ready(self)
            assert settings.DJ_CKE_IMAGE_STORAGE == "custom.image.storage"

    def test_dj_cke_permitted_image_types_default_setting(self):
        # default setting
        assert [
            "jpeg",
            "png",
            "gif",
            "bmp",
            "webp",
            "tiff",
        ] == settings.DJ_CKE_PERMITTED_IMAGE_TYPES

    def test_dj_cke_permitted_image_types_uses_user_override(self):
        # user setting
        with patch.object(settings, "DJ_CKE_PERMITTED_IMAGE_TYPES", ["xyz", "abc"]):
            DjangoCKEditorsConfig.ready(self)
            assert ["xyz", "abc"] == settings.DJ_CKE_PERMITTED_IMAGE_TYPES

    def test_dj_cke_staff_only_image_uploads_default_setting(self):
        # default setting
        assert settings.DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS is False

    def test_dj_cke_staff_only_image_uploads_uses_user_override(self):
        # user setting
        with patch.object(settings, "DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS", new=True):
            DjangoCKEditorsConfig.ready(self)
            assert settings.DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS is True

    def test_dj_cke_editors_configs_default_setting(self):
        # default setting
        keys = settings.DJ_CKE_EDITORS_CONFIGS.keys()
        assert "default" in keys
        assert "extends" in keys

    def test_dj_cke_editors_configs_uses_user_override(self):
        # user setting
        with patch.object(settings, "DJ_CKE_EDITORS_CONFIGS", {"a": 1, "b": 2}):
            DjangoCKEditorsConfig.ready(self)
            assert {"a": 1, "b": 2} == settings.DJ_CKE_EDITORS_CONFIGS



from django import forms

from django_ckeditors.forms import UploadFileForm


def test_upload_file_form_instance(upload_file_form):
    assert isinstance(upload_file_form, forms.Form)


def test_upload_file_form_field(upload_file_form):
    assert "upload" in upload_file_form.fields
    upload_field = upload_file_form.fields["upload"]
    assert isinstance(upload_field, forms.FileField)


def test_upload_file_form_invalid():
    # Create an invalid form without a file
    form = UploadFileForm(data={}, files={})
    assert not form.is_valid()
    assert "upload" in form.errors
    assert "This field is required." in form.errors["upload"]



"""django-ckeditors views."""

import logging

from django import get_version
from django.http import Http404

from django_ckeditors.exceptions import (
    InvalidImageTypeError,
    PillowImageError,
)
from django_ckeditors.helpers import has_permission_to_upload_images

if get_version() >= "4.0":
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _

from django.http import JsonResponse

from django_ckeditors.forms import UploadFileForm
from django_ckeditors.helpers import (
    handle_uploaded_image,
    image_verify,
)

logger = logging.getLogger(__name__)


def upload_image(request):

    if not has_permission_to_upload_images(request) or (
            request.method != "POST"):
        raise Http404(_("Page not found."))

    if request.method == "POST":
        form = UploadFileForm(request.POST, request.FILES)
        try:
            image_verify(request.FILES["upload"])
        except PillowImageError as e:
            return JsonResponse({"error": {"message": f"{e}"}})
        except InvalidImageTypeError as e:
            return JsonResponse({"error": {"message": f"{e}"}})
        if form.is_valid():
            url = handle_uploaded_image(request)
            return JsonResponse({"url": url})
        else:
            return JsonResponse({"error": form.errors})

    return JsonResponse({"error": {"message": "An unknown error occurred"}})



# Generated by Django 5.0.4 on 2024-04-08 23:41

import django.db.models.deletion
from django.db import migrations, models

import django_ckeditors.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Article",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(max_length=200, null=True, verbose_name="Title"),
                ),
                ("text", django_ckeditors.fields.CKEditorsField(verbose_name="Text")),
                (
                    "text2",
                    django_ckeditors.fields.CKEditorsField(
                        blank=True, null=True, verbose_name="Text 2",
                    ),
                ),
            ],
            options={
                "verbose_name": "Article",
                "verbose_name_plural": "Articles",
            },
        ),
        migrations.CreateModel(
            name="Comment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("author", models.CharField(max_length=250)),
                ("text", django_ckeditors.fields.CKEditorsField(verbose_name="Text")),
                (
                    "article",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="comments",
                        to="articles.article",
                    ),
                ),
            ],
            options={
                "verbose_name": "Comment",
                "verbose_name_plural": "Comments",
            },
        ),
    ]



import os
import shutil

from django.conf import settings

# from django.core.files.storage import Storage
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import RequestFactory, SimpleTestCase, override_settings

from django_ckeditors.helpers import (
    handle_uploaded_image,
)

# class TestHasPermissionToUploadImages:
#     @staticmethod
#     def test_non_staff_user_without_setting():
#         request = RequestFactory().get("/")
#         request.user = AnonymousUser()  # Simulate a non-authenticated user
#
#         assert (
#             has_permission_to_upload_images(request) is True
#         )  # Should have permission
#
#     @staticmethod
#     @patch("django.conf.settings.DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS", new=True)
#     def test_non_staff_user_with_setting():
#         request = RequestFactory().get("/")
#         request.user = AnonymousUser()
#
#         assert has_permission_to_upload_images(request) is False
#
#     @staticmethod
#     @patch("django.conf.settings.DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS", new=True)
#     def test_staff_user():
#         request = RequestFactory().get("/")
#         request.user = MagicMock(is_staff=True)  # Simulate staff user
#
#         assert has_permission_to_upload_images(request) is True
#
#
# class TestImageVerify:
#     @staticmethod
#     def test_valid_filetype(valid_png_image):
#
#         image_verify(image=valid_png_image)  # Should execute without raising exceptions  # noqa: E501
#
#     @staticmethod
#     def test_invalid_filetypes(invalid_jpg_image):
#         """invalid_jpg_image is a pdf file with jpg extension."""
#
#         with pytest.raises(InvalidImageTypeError):
#             image_verify(invalid_jpg_image)
#
#     @staticmethod
#     def test_corrupt_png(corrupted_png_image):
#         """corrupted_png_image is a png with random bytes changed."""
#         with pytest.raises(PillowImageError):
#             image_verify(corrupted_png_image)
#


class HandleUploadedImageTests(SimpleTestCase):
    @override_settings(
        DJ_CKE_IMAGE_STORAGE="",
        STORAGES={
            "default": {"BACKEND": "django.core.files.storage.FileSystemStorage"},
        },
    )
    def test_default_upload_url(self):
        # Create test image data
        image_data = b"Dummy image data"
        image_file = SimpleUploadedFile("test_image.jpg", image_data)

        # Create request with the test image
        request = RequestFactory().post("/", {"upload": image_file})

        result_url = handle_uploaded_image(request)
        assert result_url == settings.MEDIA_URL + "test_image.jpg"

    @override_settings(
        DJ_CKE_IMAGE_STORAGE="blog.storage.ImageCustomFileSystemStorage",
        STORAGES={
            "default": {"BACKEND": "django.core.files.storage.FileSystemStorage"},
        },
    )
    def test_image_custom_file_storage_upload(self):
        # Create test image data
        image_data = b"Dummy image data"
        image_file = SimpleUploadedFile("test_image.jpg", image_data)

        # Create request with the test image
        request = RequestFactory().post("/", {"upload": image_file})

        result_url = handle_uploaded_image(request)
        # assert 1 == 2
        assert result_url == f"tests/custom/path/{image_file.name}"

    def tearDown(self):
        """
        Overriding the TearDown function from TestCase for deleting the test file
        """

        folder = settings.MEDIA_ROOT
        for filename in os.listdir(folder):
            file_path = os.path.join(folder, filename)
            try:
                if "test" in file_path:
                    if os.path.isfile(file_path) or os.path.islink(file_path):
                        os.unlink(file_path)
                    elif os.path.isdir(file_path):

                        shutil.rmtree(file_path)
            # try:
            #     shutil.rmtree(settings.MEDIA_ROOT + "/tests/")

            except FileNotFoundError:
                pass  # File might have not been created in the test



"""Image tools."""

import io
import logging
from pathlib import Path
from weakref import proxy

# import pillow_avif
from django.core.exceptions import ValidationError
from django.core.files.images import ImageFile
from django.core.files.uploadedfile import UploadedFile
from django.utils.translation import gettext_lazy as _
from PIL import Image

logger = logging.getLogger(__name__)


def image_formatter(
    image: UploadedFile = None,
    height: int = None,
    width: int = None,
    save_format: str = 'webp',
    quality: int = None,
) -> Image.Image:
    """Take an UploadedFile image and massage it to a suitable web format.

    Params
    ======

    :param image: `django.core.files.uploadedfile.InMemoryUploadedFile` object.
    :param save_format: str Options:
                                AVIF: convert image to .avif format.
                                WEBP: convert image to .webp format.
                                JPEG: convert image to .jpg format.
                                PNG: convert image to .png format.
    :param quality: int. Optional: 0 to 100 inclusive. Worst to Best quality.


    ###############################################

    NOT IMPLEMENTED

    :param height: int. Optional: Height in pixels.
    :param width: int. Optional: Width in pixels.

    ###############################################

    :raises ValidationError:  When user attempts to upload an unsupported file.
    :raises TypeError: When an internal error caused by developer mistake
                    passing incorrect types.

    :returns: UploadedFile


    See reference material here https://github.com/trader-biz/britstralian/issues/235

    """

    # if isinstance(image, ImageFieldFile):
    #     print(f"\n\nCONVERTING IMAGE NOT REQUIRED RETURNING {image}\n\n")
    #     return image

    save_format=save_format.lower()

    match image:
        case UploadedFile():
            uploaded_file_name = Path(image.__dict__["_name"])
            # supported_image_formats = [
            #     ".jpg",
            #     ".jpeg",
            #     ".jfif",
            #     ".pjpeg",
            #     ".pjp",
            #     ".avif",
            #     ".png",
            #     ".webp",
            # ]
            #
            # # Check the uploaded file type is supported.
            # # We use a validation error here so that feedback can be supplied to the user.
            # if (
            #     not uploaded_file_name.suffix.lower()
            #     in supported_image_formats
            # ):
            #     raise ValidationError(
            #         _(
            #             "The image format %(save_format)s you have uploaded is not supported. Please use one of %(supported)s"  # noqa: E501
            #         ),
            #         params={
            #             "save_format": uploaded_file_name.suffix,
            #             "supported": str(supported_image_formats)
            #             .replace("[", "")
            #             .replace("]", "")
            #             .replace("'", "")
            #             .replace(",", ""),
            #         },
            #     )

            img_size = int(image.__dict__["size"])
            print(f'###########  IMAGE SIZE {img_size}')
            file_extension = ""

            if img_size > 20_000_000:
                raise ValidationError(
                    _(
                        "The image size %(image_size)s Megabits you have uploaded is to large. The maximum image size is 20 Megabits.",  # noqa: E501
                    ),
                    params={
                        "image_size": img_size / 1_000_000,
                    },
                )

            match save_format:
                # case None | "AVIF":
                #     save_format = "AVIF"
                #     # Determine a suitable quality for the image.
                #     # Determines the finished file size in bytes. Smaller is better.
                #     if 500_000 >= img_size:
                #         quality = 30
                #     if 500_000 <= img_size <= 1000_000:
                #         quality = 20
                #     if 1_000_001 <= img_size <= 2_000_000:
                #         quality = 10
                #     if 2_000_001 <= img_size <= 10_000_000:
                #         quality = 5
                #     if 10_000_001 <= img_size:
                #         quality = 3
                #     file_extension = ".avif"

                case 'webp' |  "jpeg" |  "png":
                    # Determine a suitable quality for the image.
                    # Determines the finished file size in bytes. Smaller is better.
                    if img_size <= 500_000:
                        quality = 30
                    if 500_000 <= img_size <= 1000_000:
                        quality = 20
                    if 1_000_001 <= img_size <= 2_000_000:
                        quality = 10
                    if 2_000_001 <= img_size <= 10_000_000:
                        quality = 5
                    if img_size >= 10_000_001:
                        quality = 3

                    file_extension = f'.{save_format}'

                # case "JPEG" | "jpeg":
                #     # Determine a suitable quality for the image.
                #     # Determines the finished file size in bytes. Smaller is better.
                #     if 500_000 >= img_size:
                #         quality = 30
                #     if 500_000 <= img_size <= 1000_000:
                #         quality = 20
                #     if 1_000_001 <= img_size <= 2_000_000:
                #         quality = 10
                #     if 2_000_001 <= img_size <= 10_000_000:
                #         quality = 5
                #     if 10_000_001 <= img_size:
                #         quality = 3
                #
                #     file_extension = f'.{save_format.lower()}'
                #
                # case "PNG" | "png":
                #     # Determine a suitable quality for the image.
                #     # Determines the finished file size in bytes. Smaller is better.
                #     if 500_000 >= img_size:
                #         quality = 30
                #     if 500_000 <= img_size <= 1000_000:
                #         quality = 20
                #     if 1_000_001 <= img_size <= 2_000_000:
                #         quality = 10
                #     if 2_000_001 <= img_size <= 10_000_000:
                #         quality = 5
                #     if 10_000_001 <= img_size:
                #         quality = 3
                #
                #     file_extension = ".png"

                case _:
                    raise TypeError(
                        f"Image save format must be of type WEBP, JPEG or PNG: {save_format} supplied",
                    )

        case _:
            raise TypeError(
                f"Image must be of type <class django.core.files.uploadedfile.UploadedFile>: {type(image)} supplied.",  # noqa: E501
            )

    # Create the new filename with correct extension to match the updated format
    file_name = Path(image.__dict__["_name"]).with_suffix(file_extension).name

    print(f'$$$$$$$ FILE NAME {file_name}\n {image}')

    # Open the image as a pillow object to change format.
    formatted_image = Image.open(image)
    # Create an in memory buffer to save the image to.
    print(f'FORMATTED IMAGE {formatted_image.size}\nSAVE FORMAT {save_format}')
    image_buffer = io.BytesIO()
    print(f'%%%%%%%  IMAGE BUFFER  {image_buffer}')

    formatted_image.save(
        image_buffer,
        format=save_format,
        quality=quality,
        optimize=True,
    )
    print(f'#####SAVED FORMATTED IMAGE {formatted_image.size}')

    updated_image = ImageFile(
        io.BytesIO(image_buffer.getvalue()),
        name=file_name,
    )

    # Close the buffer to free up memory.
    image_buffer.close()
    print(f'IMAGE STUFF ALL WORKING AND RETURING')
    # Return a Django ImageFile with updated format.
    # return updated_image


    return updated_image 



import os

import pytest

from django_ckeditors.fields import CKEditorsField
from django_ckeditors.forms import UploadFileForm


@pytest.fixture()
def valid_png_image():
    file_path = os.path.join(
        os.path.dirname(__file__),
        "fixtures",
        "files",
        "images",
        "test.png",
    )
    return open(file_path, "rb")


@pytest.fixture()
def invalid_jpg_image():
    file_path = os.path.join(
        os.path.dirname(__file__),
        "fixtures",
        "files",
        "images",
        "this_is_a_pdf_file.jpg",
    )
    return open(file_path, "rb")


@pytest.fixture()
def corrupted_png_image():
    file_path = os.path.join(
        os.path.dirname(__file__),
        "fixtures",
        "files",
        "images",
        "test_corrupted.png",
    )
    return open(file_path, "rb")


@pytest.fixture()
def ckeditors_field():
    return CKEditorsField()


@pytest.fixture()
def upload_file_form():
    return UploadFileForm()



"""test only staff can upload file"""
from django.test import override_settings
from django.urls import reverse


@override_settings(
    DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS=True,
)
def test_upload_file(admin_client, valid_png_image):
    with valid_png_image as upload:
        response = admin_client.post(
            reverse("ck_editors_upload_image"),
            {"upload": upload},
        )
    assert response.status_code == 200
    assert "url" in response.json()


@override_settings(
    DJ_CKE_STAFF_ONLY_IMAGE_UPLOADS=True,
    DJ_CKE_IMAGE_STORAGE="storages.backends.gcloud.GoogleCloudStorage",
    GS_BUCKET_NAME="test",
)
def test_upload_file_to_google_cloud(admin_client, valid_png_image, settings):
    with valid_png_image as upload:
        response = admin_client.post(
            reverse("ck_editors_upload_image"),
            {"upload": upload},
        )
    assert response.status_code == 200
    assert "url" in response.json()



"""Generate corrupted image files for testing"""
import os
import random


def corrupt_image(filename, corruption_level=0.1):
    """Corrupts a percentage of bytes in the image file and saves it with
    '_corrupted' suffix, leaving the original intact.
    """
    # Open original in read binary mode
    with open(filename, "rb") as original_file:
        image_data = original_file.read()  # Read all data into memory

    corrupted_data = bytearray(image_data)  # Create a copy for corruption

    # Corrupt the data (same logic as before)
    for _ in range(int(len(image_data) * corruption_level)):
        position = random.randint(0, len(image_data) - 1)
        corrupted_data[position] = random.randint(0, 255)

    # Save the corrupted file
    base_filename, extension = os.path.splitext(filename)
    new_filename = base_filename + "_corrupted" + extension
    with open(new_filename, "wb") as corrupted_file:
        corrupted_file.write(corrupted_data)  # Write corrupted data to new file


if __name__ == "__main__":

    # Example usage:
    corrupt_image("test.png")
